import os, random
from pyenergyplus.plugin import EnergyPlusPlugin
from pyenergyplus.api import EnergyPlusAPI
from PerformanceCurve import Curve_HPDM_SC, Curve_HPDM_SH
import time 
##############################################################################################

class SetEPAPI():
    def setEPapi(self, EP_api : EnergyPlusAPI):
        self.api = EP_api #replace API with master EnergyPlusAPI  
    

class HVACInterface(EnergyPlusPlugin, SetEPAPI):
    def __init__(self):
        super().__init__()
        self.need_to_get_handles = True
        self.IniObjects()
        
        self.Qtot_Rated = 19266.829
        self.COP_Rated =  3.384
        self.Charge_Rated = 14.0 * 20000/56996 #lbms
        self.Charge_Real = self.Charge_Rated #####to be updated by real time charge leak
        self.OAT1=0
        self.OAT2=0
        self.ZoneT1=0
        self.ZoneT2=0
        self.Qsen1=0
        self.Qsen2=0
        self.Resistance=0
        self.Capacitance=0
        self.ResistanceAvg=0
        self.CapacitanceAvg=0
        self.ResistanceMax=0
        self.CapacitanceMax=0
        self.ResistanceMin=0
        self.CapacitanceMin=0
        
        #print ("Hey outside hpdm path check: ", os.getcwd())
        #time.sleep(500)
    
# customerized codes below #################################################
    def IniObjects(self):
        # self.zone_list =['living_unit1' ,        
                        # 'attic_unit1' ]
                        
        self.HP_SC = Curve_HPDM_SC()
        
    def get_sensor_handles(self, state):
        self.OAT_sensor_handle = self.api.exchange.get_variable_handle(state,
                                        "Site Outdoor Air Drybulb Temperature", "Environment")

        print ('\n ###################', self.OAT_sensor_handle, '\n ####################')

        self.ZoneT_Ctrl_Zone_sensor_handle = self.api.exchange.get_variable_handle(state,
                                        "Zone Mean Air Temperature", "living_unit1")

        print ('\n ###################', self.ZoneT_Ctrl_Zone_sensor_handle, '\n ####################')

        self.ZoneR_Ctrl_Zone_sensor_handle = self.api.exchange.get_variable_handle(state,
                                        "Zone Air Relative Humidity", "living_unit1")

        print ('\n ###################', self.ZoneR_Ctrl_Zone_sensor_handle, '\n ####################')


        self.Fan_Massflow_Ctrl_Zone_sensor_handle =  self.api.exchange.get_variable_handle(state,
                                        "Fan Air Mass Flow Rate", "Supply Fan_unit1")

        print('\n ###################', self.Fan_Massflow_Ctrl_Zone_sensor_handle, '\n ####################')
        
        self.Coil_TotCoolRate_handle = self.api.exchange.get_variable_handle(state,
                                        "Cooling Coil Total Cooling Rate", "DX COOLING COIL_UNIT1")
        
        print('\n ###################', self.Coil_TotCoolRate_handle, '\n ####################')
        
        self.Coil_SenCoolRate_handle = self.api.exchange.get_variable_handle(state,
                                        "Cooling Coil Sensible Cooling Rate", "DX COOLING COIL_UNIT1")
        
        print('\n ###################', self.Coil_SenCoolRate_handle, '\n ####################')
                
        #time. sleep(5)

        return 0
        
    def get_actuator_handles(self, state):
        ############## performance curve handles ##############                                                                                

        self.Qdot_HP_SC_actuator_handle = self.api.exchange.get_actuator_handle(state,
                                    component_type= "Curve",
                                    control_type='Curve Result',
                                    actuator_key='HPACCoolCapFT')  
                                    
                                    
        self.EIR_HP_SC_actuator_handle = self.api.exchange.get_actuator_handle(state,
                                    component_type= "Curve",
                                    control_type='Curve Result',
                                    actuator_key='HPACCOOLEIRFT')   
                  
        self.Qdot_HP_FlowFrac_actuator_handle = self.api.exchange.get_actuator_handle(state,
                                    component_type= "Curve",
                                    control_type='Curve Result',
                                    actuator_key='HPACCoolCapFFF') 
                                    
        self.EIR_HP_FlowFrac_actuator_handle = self.api.exchange.get_actuator_handle(state,
                                    component_type= "Curve",
                                    control_type='Curve Result',
                                    actuator_key='HPACCOOLEIRFFF')      
               
        self.PLR_SC_actuator_handle = self.api.exchange.get_actuator_handle(state,
                                    component_type= "Curve",
                                    control_type='Curve Result',
                                    actuator_key='HPACCOOLPLFFPLR')                              
        
        return 0
      
    def on_end_of_system_timestep_after_hvac_reporting(self, state) -> int:
    
        if self.need_to_get_handles==False:                
            self.OAT2=self.OAT1
            self.ZoneT2=self.ZoneT1
            self.Qsen2=self.Qsen1
            self.OAT1=self.api.exchange.get_variable_value(state, self.OAT_sensor_handle) #C
            self.ZoneT1=self.api.exchange.get_variable_value(state, self.ZoneT_Ctrl_Zone_sensor_handle) #C
            self.Qsen1=self.api.exchange.get_variable_value(state, self.Coil_SenCoolRate_handle)
            print('\n ###################', self.Qsen1, '\n ####################')
            #solve RC model based on last two timesteps
            # self.Resistance=0
            # self.Capacitance=0
            # self.ResistanceAvg=0
            # self.CapacitanceAvg=0
            # self.ResistanceMax=0
            # self.CapacitanceMax=0
            # self.ResistanceMin=0
            # self.CapacitanceMin=0
        
        return 0
        
    def on_begin_timestep_before_predictor(self, state) -> int:
        if self.api.exchange.api_data_fully_ready(state):
            if self.need_to_get_handles:
                self.get_sensor_handles(state)
                self.get_actuator_handles(state)
                self.need_to_get_handles = False
                
            DayLeak : int = self.api.exchange.day_of_year(state)
            
            if DayLeak < 100:
                self.Charge_Real=self.Charge_Rated
            else:
                self.Charge_Real=self.Charge_Rated - (self.Charge_Rated * (DayLeak - 100) /(365 - 100) * 0.5)
                
            ChargeRatioInput = self.Charge_Real / self.Charge_Rated 
            #ChargeRatioInput = 0.7

            #print('\n ###################', self.api.exchange.day_of_year(state), '\n ####################')
            
            # get some variables from building, whcih are input to HPDM
            # C to F
            OAT = self.api.exchange.get_variable_value(state, self.OAT_sensor_handle) * 1.8 +32
            # C to F
            ZoneT_Ctrl = self.api.exchange.get_variable_value(state, self.ZoneT_Ctrl_Zone_sensor_handle)* 1.8 +32
            
            # humidity % to -
            ZoneR_Ctrl = self.api.exchange.get_variable_value(state, self.ZoneR_Ctrl_Zone_sensor_handle)*0.01
            # MFR kg/s to lbs/hour
            #SAMass_Ctrl = self.api.exchange.get_variable_value(state, self.Fan_Massflow_Ctrl_Zone_sensor_handle) *7936.64
            
            self.HP_SC.get_input_variables( cooling_cap_rated=self.Qtot_Rated, air_temperature_indoor=ZoneT_Ctrl, air_rh_indoor=ZoneR_Ctrl, air_temperature_outdoor=OAT, ChargeRatio = ChargeRatioInput )
            (Qtot_Ctrl, Qsen_Ctrl, Power_Ctrl) = self.HP_SC.get_output_variable()
            print ('Core zone HP results : ', Qtot_Ctrl, Qsen_Ctrl, Power_Ctrl)
                        
            if Power_Ctrl<1e-3:
                COP_Ctrl =0.001
            else:
                COP_Ctrl = Qtot_Ctrl/Power_Ctrl/3.412
                          
            print('\n ###################', COP_Ctrl, '\n ####################')
            
            curve_Qdot_HP_SC = Qtot_Ctrl/self.Qtot_Rated
            
            print('\n ###################', curve_Qdot_HP_SC, '\n ####################')

            self.api.exchange.set_actuator_value(state, self.Qdot_HP_SC_actuator_handle, curve_Qdot_HP_SC )

            curve_Qdot_HP_FlowFrac = 1
            self.api.exchange.set_actuator_value(state, self.Qdot_HP_FlowFrac_actuator_handle, curve_Qdot_HP_FlowFrac )
                      
            curve_EIR_HP_SC = self.COP_Rated/COP_Ctrl        
            self.api.exchange.set_actuator_value(state, self.EIR_HP_SC_actuator_handle, curve_EIR_HP_SC )

            curve_EIR_HP_FlowFrac = 1 
            self.api.exchange.set_actuator_value(state, self.EIR_HP_FlowFrac_actuator_handle, curve_EIR_HP_FlowFrac )
          
                
        return 0
    

