import os, random
import sys
sys.path.append(os.environ['HPDMFLEX'])
import pyHPDM.api as HPDM

class CallHPDM():
    def SetInputs(self):
        error : int = 0
        #os.chdir(self.data_path)  
        for HPInput in self.Inputs:
            error = error + self.HPDMapi.IN(HPInput)   
        #os.chdir(self.current_path)
        return error
    
    def RunAndOutputs(self):
        # this function will call the HPDM backend to calculate the desired total cooling, sensible cooling, and power consumption 
        os.chdir(self.data_path)  
        error : int = self.HPDMapi.RUN() 
        if(error == 0):
            for HPOutput in self.Outputs:
                self.HPDMapi.OUT(HPOutput)        
        os.chdir(self.current_path)
        return error
    
    def InitPath(self):
        self.current_path = os.getcwd() # energyplus temporary folder created by EnergyPlus
        os.chdir("..\..\HP_DXVarCharg")        
        self.data_path = os.getcwd() #HPDM input data files     
        print ("hey initial path: ", os.getcwd())
        # time.sleep(500)
        #sys.("pause")
        os.chdir(self.current_path)
        #print ("final check path: ", os.getcwd())
        #print ("I am here")
        #time.sleep(500)

#custimized codes below ###############################################
class Curve_HPDM_SC(CallHPDM):
    def __init__(self):    
        self.InitPath()
        
        #define HPDM instance, iputs and outputs
        self.str1 = "SystemConfig.dat"
        self.iObjID = 0              
        self.iRunMode = 0
        self.iPrintFlg = 0
        self.iDebug = 0
        self.HPDMapi = HPDM.HPDMInstance(self.str1, self.iObjID, self.iRunMode, self.iPrintFlg, self.iDebug, isNewDLL = False)
        
        #inputs
        self.iCompID = 0   
        self.dRatCapacity=HPDM.HPDMInput(self.iCompID, HPDM.NAt.V, 0, 20000.0)
        self.INDTemp = HPDM.HPDMInput(self.iCompID, HPDM.NAt.V, 1, 80.0) 
        self.INDRH = HPDM.HPDMInput(self.iCompID, HPDM.NAt.V, 2, 0.5)  
        self.Tamb = HPDM.HPDMInput(self.iCompID, HPDM.NAt.V, 3, 95.0) 
        self.ChargR = HPDM.HPDMInput(self.iCompID, HPDM.NAt.V, 4, 1.0) 
        self.Inputs : HPDM.HPDMInput = {self.dRatCapacity, self.INDTemp, self.INDRH, self.Tamb, self.ChargR}
        
        #outputs
        self.sQtot = HPDM.HPDMOutput(self.iCompID, 11)
        self.sQsen = HPDM.HPDMOutput(self.iCompID, 12)
        self.sPower =HPDM.HPDMOutput(self.iCompID, 13)        
        self.Outputs : HPDM.HPDMOutput ={self.sQtot, self.sQsen, self.sPower}
        
    
    def get_input_variables(self, cooling_cap_rated=10000, air_temperature_indoor=80, air_rh_indoor=0.5, air_temperature_outdoor=95, ChargeRatio = 1.0):
        # cooling_cap_rated: cooling capacity in watt, (default=10000 watt) 
        # air_temperature_indoor: indoor air temperature in Farenheite (default = 80 F)
        # air_rh_indoor: indoor air relative humidity, in fraction [0, 1] (default =0.5)
        # air_temperature_outdoor: outdoor air temperature in Fareiheite  (default=100F)
        
        self.dRatCapacity.Value = cooling_cap_rated
        self.INDTemp.Value = air_temperature_indoor
        self.INDRH.Value = air_rh_indoor
        self.Tamb.Value = air_temperature_outdoor
        self.ChargR.Value = ChargeRatio
        
        error : int = self.SetInputs()        
        return error
        

        
    def get_output_variable(self):
        error :int = self.RunAndOutputs()
                        
        print(self.sQtot.Value)
        print(self.sQsen.Value)
        print(self.sPower.Value)
        #time.sleep(500)
                
        return (self.sQtot.Value, self.sQsen.Value, self.sPower.Value)

class Curve_HPDM_SH(CallHPDM):
    def __init__(self):    
        self.InitPath()
        
        #define HPDM instance, iputs and outputs
        self.str1 = "SystemConfig1.dat"
        self.iObjID = 0              
        self.iRunMode = 0
        self.iPrintFlg = 0
        self.iDebug = 0
        self.HPDMapi = HPDM.HPDMInstance(self.str1, self.iObjID, self.iRunMode, self.iPrintFlg, self.iDebug, isNewDLL = False)
        
        #inputs
        self.iCompID = 0   
        self.dRatCapacity=HPDM.HPDMInput(self.iCompID, HPDM.NAt.V, 0, 10000.0)
        self.INDTemp = HPDM.HPDMInput(self.iCompID, HPDM.NAt.V, 1, 80.0) 
        self.INDRH = HPDM.HPDMInput(self.iCompID, HPDM.NAt.V, 2, 0.5)  
        self.Tamb = HPDM.HPDMInput(self.iCompID, HPDM.NAt.V, 3, 95.0) 
        self.ChargR = HPDM.HPDMInput(self.iCompID, HPDM.NAt.V, 4, 1.0) 
        self.Inputs : HPDM.HPDMInput = {self.dRatCapacity, self.INDTemp, self.INDRH, self.Tamb, self.ChargR}
        
        #outputs
        self.sQtot = HPDM.HPDMOutput(self.iCompID, 11)
        self.sQsen = HPDM.HPDMOutput(self.iCompID, 12)
        self.sPower =HPDM.HPDMOutput(self.iCompID, 13)        
        self.Outputs : HPDM.HPDMOutput ={self.sQtot, self.sQsen, self.sPower}
        
    
    def get_input_variables(self, cooling_cap_rated=10000, air_temperature_indoor=80, air_rh_indoor=0.5, air_temperature_outdoor=95, ChargeRatio = 1.0):
        # cooling_cap_rated: cooling capacity in watt, (default=10000 watt) 
        # air_temperature_indoor: indoor air temperature in Farenheite (default = 80 F)
        # air_rh_indoor: indoor air relative humidity, in fraction [0, 1] (default =0.5)
        # air_temperature_outdoor: outdoor air temperature in Fareiheite  (default=100F)
        
        self.dRatCapacity.Value = cooling_cap_rated
        self.INDTemp.Value = air_temperature_indoor
        self.INDRH.Value = air_rh_indoor
        self.Tamb.Value = air_temperature_outdoor
        self.ChargR.Value = ChargeRatio
        
        error : int = self.SetInputs()        
        return error
        

        
    def get_output_variable(self):
        error :int = self.RunAndOutputs()
                        
        print(self.sQtot.Value)
        print(self.sQsen.Value)
        print(self.sPower.Value)
        #time.sleep(500)
                
        return (self.sQtot.Value, self.sQsen.Value, self.sPower.Value)